@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2007-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -10,7 +10,7 @@
 # ABSTRACT: Edit the configuration of an application
 
 package App::Cme::Command::meta ;
-$App::Cme::Command::meta::VERSION = '2.002';
+$App::Cme::Command::meta::VERSION = '2.004';
 use strict ;
 use warnings ;
 use 5.10.1;
@@ -360,19 +360,24 @@ sub _edit {
     $cmu->after(10, sub { $cmu->force_element_display($obj) });
 
     if (my $taq = $opt->test_and_quit ) {
-        $cmu->after( 1000 , sub {
+        my $bail_out = sub {
+            warn "save failed: $_[0]\n" if @_;
+            $cmu -> quit;
+        } ;
+
+        $cmu->after( 2000 , sub {
             if ($taq =~ /s/) {
                 say "Test mode: save and quit";
-               $cmu->save( sub {$cmu -> quit;});
+               $cmu->save( $bail_out );
             }
             else {
                 say "Test mode: quit only";
-                $cmu->quit;
+                &$bail_out
             }
         });
     }
     &MainLoop ; # Tk's
-
+    say "Exited GUI";
 }
 
 1;
@@ -389,7 +394,7 @@ App::Cme::Command::meta - Edit the configuration of an application
 
 =head1 VERSION
 
-version 2.002
+version 2.004
 
 =head1 SYNOPSIS
 
@@ -450,7 +455,7 @@ For instance:
 
 =head2 gen-dot [ file.dot ]
 
-Create a dot file that represent the stucture of the configuration
+Create a dot file that represent the structure of the configuration
 model. By default, the generated dot file is C<model.dot>
 
  $ cme meta gen-dot Itself itself.dot
@@ -623,7 +628,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2007-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
