@@ -1,16 +1,16 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2007-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Itself ;
-$Config::Model::Itself::VERSION = '2.002';
+$Config::Model::Itself::VERSION = '2.004';
 use Mouse ;
-use Config::Model 2.064;
+use Config::Model 2.083;
 use 5.010;
 
 use IO::File ;
@@ -32,8 +32,95 @@ coerce 'ModelPathTiny'  => from 'Str'  => via {path($_)} ;
 
 # find all .pl file in model_dir and load them...
 
-has model_object => (is =>'ro', isa =>'Config::Model::Node', required => 1) ;
-has cm_lib_dir    => (is =>'ro', isa => 'ModelPathTiny', required => 1, coerce => 1 ) ;
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my %args  = @_;
+
+    my $legacy = delete $args{model_object};
+    if ($legacy) {
+        $args{config_model} = $legacy->instance->config_model;
+        $args{meta_instance} = $legacy->instance;
+        $args{meta_root} = $legacy;
+    }
+    return $class->$orig( %args );
+};
+
+has 'config_model' => (
+    is => 'ro',
+    isa => 'Config::Model',
+    lazy_build => 1,
+) ;
+
+
+sub _build_config_model {
+    my $self = shift;
+    # don't trigger builders below
+    if ($self->{meta_root}) {
+        return $self->meta_root->instance->config_model;
+    }
+    elsif ($self->{meta_instance}) {
+        return $self->meta_instance->config_model;
+    }
+    else {
+        return Config::Model -> new ( ) ;
+    }
+}
+
+has check        => (is =>'ro', isa => 'Bool', default => 1) ;
+
+has 'meta_instance' => (
+    is =>'ro',
+    isa =>'Config::Model::Instance',
+    lazy_build => 1,
+) ;
+
+sub _build_meta_instance {
+    my $self = shift;
+
+    # don't trigger builders below
+    if ($self->{meta_root}) {
+        return $self->meta_root->instance;
+    }
+    else {
+        # load Config::Model model
+        return $self->config_model->instance (
+            root_class_name => 'Itself::Model' ,
+            instance_name   => 'meta_model' ,
+            check => $self->check,
+        );
+    }
+
+}
+
+has meta_root => (
+    is =>'ro',
+    isa =>'Config::Model::Node',
+    lazy_build => 1,
+) ;
+
+sub _build_meta_root {
+    my $self = shift;
+
+    return $self->meta_instance -> config_root ;
+}
+
+has cm_lib_dir   => (
+    is =>'ro',
+    isa => 'ModelPathTiny',
+    lazy_build => 1,
+    coerce => 1
+) ;
+
+sub _build_cm_lib_dir {
+    my $self = shift;
+    my $p =  path('lib/Config/Model');
+    if (! $p->is_dir) {
+        $p->mkpath(0, 0755) || die "can't create $p:$!";
+    }
+    return $p;
+}
+
 has force_write  => (is =>'ro', isa => 'Bool', default => 0) ;
 has root_model   => (is =>'ro', isa => 'str');
 
@@ -46,7 +133,7 @@ has modified_classes => (
         clear_classes => 'clear',
         set_class => 'set',
         class_was_changed => 'get' ,
-        classes_to_write => 'keys' ,
+        class_known => 'exists',
     }
 ) ;
 
@@ -70,21 +157,31 @@ sub BUILD {
         my %args = @_ ;
         my $p = $args{path} || '' ;
         return unless $p =~ /^class/ ;
+        return unless $args{index}; # may be empty when class order is changed
         return if $self->class_was_changed($args{index}) ;
         $logger->info("class $args{index} was modified");
 
         $self->add_modified_class($args{index}) ;
     } ;
-    $self->model_object->instance -> on_change_cb($cb) ;
+    $self->meta_instance -> on_change_cb($cb) ;
 
 }
 
+sub add_tracked_class {
+    my $self = shift;
+    $self->set_class(shift,0) ;
+}
 
 sub add_modified_class {
     my $self = shift;
     $self->set_class(shift,1) ;
 }
 
+sub class_needs_write {
+    my $self = shift;
+    my $name =  shift;
+    return ($self->force_write or not $self->class_known($name) or $self->class_was_changed($name)) ;
+}
 
 sub read_app_files {
     my $self = shift;
@@ -119,7 +216,7 @@ sub read_app_files {
             my $appli = $file->basename;
             $apps{$appli} = $data{model} ;
 
-            $self->model_object->load_data(
+            $self->meta_root->load_data(
                 data => { application => { $appli => \%data } },
                 check => $force_load ? 'no' : 'yes'
             ) ;
@@ -167,7 +264,7 @@ sub read_all {
     } ;
     $read_dir->visit($wanted, { recurse => 1} ) ;
 
-    my $i = $self->model_object->instance ;
+    my $i = $self->meta_instance ;
 
     my %read_models ;
     my %pod_data ;
@@ -197,6 +294,9 @@ sub read_all {
             my $raw_model =  $tmp_model -> get_raw_model( $model_name ) ;
             my $new_model =  $tmp_model -> get_model( $model_name ) ;
 
+            # track read class to identify later classes added by user
+            $self->add_tracked_class($model_name);
+
             # some modifications may be done to cope with older model styles. If a modif
             # was done, mark the class as changed so it will be saved later
             $self->add_modified_class($model_name) unless Compare($raw_model, $new_model) ;
@@ -236,15 +336,15 @@ sub read_all {
 
     # Create all classes listed in %read_models to avoid problems with
     # include statement while calling load_data
-    my $model_obj = $self->model_object ;
-    my $class_element = $model_obj->fetch_element('class') ;
+    my $root_obj = $self->meta_root ;
+    my $class_element = $root_obj->fetch_element('class') ;
     map { $class_element->fetch_with_id($_) } sort keys %read_models ;
 
     #require Tk::ObjScanner; Tk::ObjScanner::scan_object(\%read_models) ;
 
     $logger->info("loading all extracted data in Config::Model::Itself");
     # load with a array ref to avoid warnings about missing order
-    $model_obj->load_data(
+    $root_obj->load_data(
         data => {class => [ %read_models ] },
         check => $force_load ? 'no' : 'yes'
     ) ;
@@ -255,7 +355,7 @@ sub read_all {
         my $fh = IO::File->new($file) || die "Can't open $file: $!" ;
         my @lines = $fh->getlines ;
         $fh->close;
-        $model_obj->load_pod_annotation(join('',@lines)) ;
+        $root_obj->load_pod_annotation(join('',@lines)) ;
 
         my @headers ;
         foreach my $l (@lines) {
@@ -278,11 +378,11 @@ sub read_all {
 sub get_perl_data_model{
     my $self = shift ;
     my %args = @_ ;
-    my $model_obj = $self->{model_object};
+    my $root_obj = $self->{meta_root};
     my $class_name = $args{class_name}
       || croak __PACKAGE__," read: undefined class name";
 
-    my $class_element = $model_obj->fetch_element('class') ;
+    my $class_element = $root_obj->fetch_element('class') ;
 
     # skip if class was deleted during edition
     return unless $class_element->defined($class_name) ;
@@ -305,7 +405,7 @@ sub write_app_files {
     my $self = shift;
 
     my $app_dir = $self->cm_lib_dir;
-    my $app_obj = $self->model_object->fetch_element('application');
+    my $app_obj = $self->meta_root->fetch_element('application');
 
     foreach my $app_name ( $app_obj->fetch_all_indexes ) {
         my $app = $app_obj->fetch_with_id($app_name);
@@ -331,7 +431,7 @@ sub write_app_files {
 sub write_all {
     my $self = shift ;
     my %args = @_ ;
-    my $model_obj = $self->model_object ;
+    my $root_obj = $self->meta_root ;
     my $dir = $self->model_dir ;
 
     croak "write_all: unexpected parameters ",join(' ', keys %args) if %args ;
@@ -345,7 +445,7 @@ sub write_all {
     # get list of all classes loaded by the editor
     my %loaded_classes
       = map { ($_ => 1); }
-        $model_obj->fetch_element('class')->fetch_all_indexes ;
+        $root_obj->fetch_element('class')->fetch_all_indexes ;
 
     # remove classes that are listed in map
     foreach my $file (keys %$map) {
@@ -372,7 +472,7 @@ sub write_all {
 
         # check if any a class of a file was modified
         foreach my $class_name (@{$map_to_write{$file}}) {
-            $file_needs_write++ if $self->force_write or $self->class_was_changed($class_name) ;
+            $file_needs_write++ if $self->class_needs_write($class_name);
             $logger->info("file $file class $class_name needs write ",$file_needs_write);
         }
 
@@ -384,7 +484,7 @@ sub write_all {
               = $self-> get_perl_data_model(class_name => $class_name) ;
             push @data, $model if defined $model and keys %$model;
 
-            my $node = $self->{model_object}->grab("class:".$class_name) ;
+            my $node = $self->{meta_root}->grab("class:".$class_name) ;
             push @notes, $node->dump_annotations_as_pod ;
             # remove class name from above list
             delete $loaded_classes{$class_name} ;
@@ -395,7 +495,7 @@ sub write_all {
         write_model_file ($dir->child($file), $self->{header}{$file}, \@notes, \@data);
     }
 
-    $self->model_object->instance->clear_changes ;
+    $self->meta_instance->clear_changes ;
 }
 
 sub write_model_snippet {
@@ -407,7 +507,7 @@ sub write_model_snippet {
       || croak __PACKAGE__," write_model_snippet: undefined model_file";
     croak "write_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
 
-    my $model = $self->model_object->dump_as_data ;
+    my $model = $self->meta_root->dump_as_data ;
     # print (Dumper( $model)) ;
 
     my @raw_data = @{$model->{class} || []} ;
@@ -416,13 +516,13 @@ sub write_model_snippet {
         $data ->{name} = $class ;
 
         # does not distinguish between notes from underlying model or snipper notes ...
-        my @notes = $self->model_object->grab("class:$class")->dump_annotations_as_pod ;
+        my @notes = $self->meta_root->grab("class:$class")->dump_annotations_as_pod ;
         my $class_dir = $class.'.d';
         $class_dir =~ s!::!/!g;
         write_model_file ("$snippet_dir/$class_dir/$model_file", [], \@notes, [ $data ]);
     }
 
-    $self->model_object->instance->clear_changes ;
+    $self->meta_instance->clear_changes ;
 }
 
 sub read_model_snippet {
@@ -446,7 +546,7 @@ sub read_model_snippet {
     } ;
     find ($wanted, $snippet_dir ) ;
 
-    my $class_element = $self->model_object->fetch_element('class') ;
+    my $class_element = $self->meta_root->fetch_element('class') ;
 
     foreach my $load_file (@files) {
         $logger->info("trying to read snippet $load_file");
@@ -471,7 +571,7 @@ sub read_model_snippet {
         my $fh = IO::File->new($load_file) || die "Can't open $load_file: $!" ;
         my @lines = $fh->getlines ;
         $fh->close;
-        $self->model_object->load_pod_annotation(join('',@lines)) ;
+        $self->meta_root->load_pod_annotation(join('',@lines)) ;
     }
 }
 
@@ -520,7 +620,7 @@ sub list_class_element {
     my $pad  =  shift || '' ;
 
     my $res = '';
-    my $meta_class = $self->{model_object}->fetch_element('class') ;
+    my $meta_class = $self->{meta_root}->fetch_element('class') ;
     foreach my $class_name ($meta_class->fetch_all_indexes ) {
         $res .= $self->list_one_class_element($class_name) ;
     }
@@ -533,7 +633,7 @@ sub list_one_class_element {
     my $pad  =  shift || '' ;
 
     my $res = $pad."Class: $class_name\n";
-    my $meta_class = $self->{model_object}->fetch_element('class')
+    my $meta_class = $self->{meta_root}->fetch_element('class')
        -> fetch_with_id($class_name) ;
 
     my @elts = $meta_class->fetch_element('element')->fetch_all_indexes ;
@@ -563,7 +663,7 @@ sub get_dot_diagram {
     my $self = shift ;
     my $dot = "digraph model {\n" ;
 
-    my $meta_class = $self->{model_object}->fetch_element('class') ;
+    my $meta_class = $self->{meta_root}->fetch_element('class') ;
     foreach my $class_name ($meta_class->fetch_all_indexes ) {
         my $d_class = $class_name ;
         $d_class =~ s/::/__/g;
@@ -571,7 +671,7 @@ sub get_dot_diagram {
         my $elt_list = '';
         my $use = '';
 
-        my $class_obj =  $self->{model_object}->grab(qq!class:"$class_name"!);
+        my $class_obj =  $self->{meta_root}->grab(qq!class:"$class_name"!);
         my @elts =  $class_obj ->grab(qq!element!) ->fetch_all_indexes ;
         foreach my $elt_name ( @elts ) {
             my $of = '';
@@ -660,32 +760,19 @@ Config::Model::Itself - Model editor for Config::Model
 
 =head1 VERSION
 
-version 2.002
+version 2.004
 
 =head1 SYNOPSIS
 
- my $meta_model = Config::Model -> new ( ) ;
-
- # load Config::Model model
- my $meta_inst = $meta_model->instance (
-    root_class_name => 'Itself::Model' ,
-    instance_name   => 'meta_model' ,
- );
-
- my $meta_root = $meta_inst -> config_root ;
-
  # Itself constructor returns an object to read or write the data
  # structure containing the model to be edited
- my $rw_obj = Config::Model::Itself -> new(
-    model_object => $meta_root,
-    model_dir => '/path/to/model_files', # can be a Path::Tiny object
- ) ;
+ my $meta_model = Config::Model::Itself -> new( ) ;
 
  # now load the model to be edited
- $rw_obj -> read_all( ) ;
+ $meta_model -> read_all( ) ;
 
  # For Curses UI prepare a call-back to write model
- my $wr_back = sub { $rw_obj->write_all(); }
+ my $wr_back = sub { $meta_model->write_all(); }
 
  # create Curses user interface
  my $dialog = Config::Model::CursesUI-> new (
@@ -693,7 +780,7 @@ version 2.002
  ) ;
 
  # start Curses dialog to edit the mode
- $dialog->start( $meta_model )  ;
+ $dialog->start( $meta_model->config_root )  ;
 
  # that's it. When user quits curses interface, Curses will call
  # $wr_back sub ref to write the modified model.
@@ -725,16 +812,23 @@ dedicated to read and write a set of model files.
 
 =head1 Constructor
 
-=head2 new ( model_object => ... , model_dir => ... )
+=head2 new ( [ cm_lib_dir => ... ] )
 
-Creates a new read/write handler. This handler is dedicated to the
-C<model_object> passed with the constructor. This parameter must be a
-L<Config::Model::Node> class. C<model_dir> is either a C<Path::Tiny> object
-or a string.
+Creates a new read/write handler. If no model_object is passed, the required
+objects are created. C<cm_lib_dir> specifies where are the model files (defaults to
+C<./lib/Config/Model>.
+
+C<cm_lib_dir> is either a C<Path::Tiny> object or a string.
+
+By default, this constructor will create all necessary
+C<Config::Model*> objects.  If needed, you can pass already created
+object with options C<config_model> (L<Config::Model> object),
+C<meta_instance> (L<Config::Model::Instance> object) or C<meta_root>
+(L<Config::Model::Node> object).
 
 =head2 Methods
 
-=head1 read_all (  root_model => ... , [ force_load => 1 ] )
+=head1 read_all ( [ root_model => ... ], [ force_load => 1 ] )
 
 Load all the model files contained in C<model_dir> and all its
 subdirectories. C<root_model> is used to filter the classes read.
@@ -766,14 +860,14 @@ debugging your configuration model.
 
 =head2 get_dot_diagram
 
-Returns a graphviz dot file that represents the strcuture of the
+Returns a graphviz dot file that represents the structure of the
 configuration model:
 
 =over
 
 =item *
 
-C<include> are represented by solid lines
+C<include> relations are represented by solid lines
 
 =item *
 
@@ -782,6 +876,13 @@ dashed lines. The name of the element is attached to the dashed line.
 
 =back
 
+=head1 BUGS
+
+Test menu entries are created from the content of C<application> model
+parameter.  Unfortunately, there's no way to build the menu
+dynamically. So user cme must be restarted to change the menu if the
+application list is changed.
+
 =head1 SEE ALSO
 
 L<Config::Model>, L<Config::Model::Node>, L<Path::Tiny>
@@ -792,10 +893,95 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2007-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
   The GNU Lesser General Public License, Version 2.1, February 1999
 
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+Search CPAN
+
+The default CPAN search engine, useful to view POD in HTML format.
+
+L<http://search.cpan.org/dist/Config-Model-Itself>
+
+=item *
+
+AnnoCPAN
+
+The AnnoCPAN is a website that allows community annotations of Perl module documentation.
+
+L<http://annocpan.org/dist/Config-Model-Itself>
+
+=item *
+
+CPAN Ratings
+
+The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
+
+L<http://cpanratings.perl.org/d/Config-Model-Itself>
+
+=item *
+
+CPANTS
+
+The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
+
+L<http://cpants.perl.org/dist/overview/Config-Model-Itself>
+
+=item *
+
+CPAN Testers
+
+The CPAN Testers is a network of smokers who run automated tests on uploaded CPAN distributions.
+
+L<http://www.cpantesters.org/distro/C/Config-Model-Itself>
+
+=item *
+
+CPAN Testers Matrix
+
+The CPAN Testers Matrix is a website that provides a visual overview of the test results for a distribution on various Perls/platforms.
+
+L<http://matrix.cpantesters.org/?dist=Config-Model-Itself>
+
+=item *
+
+CPAN Testers Dependencies
+
+The CPAN Testers Dependencies is a website that shows a chart of the test results of all dependencies for a distribution.
+
+L<http://deps.cpantesters.org/?module=Config::Model::Itself>
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<ddumont at cpan.org>, or through
+the web interface at L<https://github.com/dod38fr/config-model-itself/issues>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
+
+The code is open to the world, and available for you to hack on. Please feel free to browse it and play
+with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
+from your repository :)
+
+L<http://github.com/dod38fr/config-model-itself>
+
+  git clone git://github.com/dod38fr/config-model-itself.git
+
 =cut
