@@ -1,14 +1,18 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2007-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Itself::BackendDetector ;
-$Config::Model::Itself::BackendDetector::VERSION = '2.002';
+$Config::Model::Itself::BackendDetector::VERSION = '2.004';
+# since this package is mostly targeted for dev environments
+# let the detector detect models under development
+use lib 'lib';
+
 use Pod::POM ;
 use File::Find ;
 
@@ -39,7 +43,7 @@ sub setup_enum_choice {
         my $path = "$inc/Config/Model/Backend" ;
         find ($wanted, $path ) if -d $path;
     }
-    
+
     $self->SUPER::setup_enum_choice(sort keys %choices) ;
 }
 
@@ -55,26 +59,26 @@ sub set_help {
 
     my $wanted = sub { 
         my $n = $File::Find::name ;
-	
+
         return unless (-f $n and $n !~ /Any\.pm$/) ;
         my $file = $n ;
-	$n =~ s/\.pm$//;
-	$n =~ s!/!::!g ;
-	my $perl_name = $n ;
-	$n =~ s!.*Backend::!! ;
-	$perl_name =~ s!.*Config!Config! ;
+        $n =~ s/\.pm$//;
+        $n =~ s!/!::!g ;
+        my $perl_name = $n ;
+        $n =~ s!.*Backend::!! ;
+        $perl_name =~ s!.*Config!Config! ;
 
-	my $pom = $parser->parse_file($file)|| die $parser->error();
+        my $pom = $parser->parse_file($file)|| die $parser->error();
 
-	foreach my $head1 ($pom->head1()) {
-	    if ($head1->title() eq 'NAME') {
-		my $c = $head1->content();
-		$c =~ s/.*?-\s*//;
-		$c =~ s/\n//g;
-		$help->{$n} = $c . " provided by $perl_name"."::".$n;
-		last;
-	    }
-	}
+        foreach my $head1 ($pom->head1()) {
+            if ($head1->title() eq 'NAME') {
+                my $c = $head1->content();
+                $c =~ s/.*?-\s*//;
+                $c =~ s/\n//g;
+                $help->{$n} = $c . " provided by L<$perl_name>";
+                last;
+            }
+        }
     };
 
     find ($wanted, $path ) ;
@@ -98,7 +102,7 @@ Config::Model::Itself::BackendDetector - Detect available read/write backends us
 
 =head1 VERSION
 
-version 2.002
+version 2.004
 
 =head1 SYNOPSIS
 
@@ -155,7 +159,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dominique Dumont.
+This software is Copyright (c) 2007-2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
