@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2015 by Dominique Dumont.
+# This software is Copyright (c) 2007-2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -35,18 +35,22 @@
                 type       => 'leaf',
                 level      => 'hidden',
                 value_type => 'enum',
+                choice => [
+                    qw/boolean enum integer reference
+                       number uniline string file dir/
+                ],
                 'warp'     => {
                     follow  => { 't' => '- type' },
                     'rules' => [
                         '$t eq "leaf"' => {
-                            choice => [
-                                qw/boolean enum integer reference
-                                  number uniline string file dir/
-                            ],
                             level     => 'normal',
                             mandatory => 1,
                         }
                     ]
+                },
+                help => {
+                    integer => 'positive or negative integer',
+                    uniline => 'string with no embedded newline',
                 }
             },
 
